# -*- coding: utf-8 -*-
# cython: language_level=3

__all__ = [
    "APP_NAME",
    "PACKAGE_NAME",
    "ORGANIZATION_NAME",
    "VERSION",
]


# application information
APP_NAME = "Scaldys"

# packaging information
PACKAGE_NAME = "scaldys"  # lower case
ORGANIZATION_NAME = "Scaldys"

# version information
VERSION = "0.1.1"
